"
 COPYRIGHT (c) 1995 by Claus Gittinger
	      All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"

Object subclass:#AbstractSourceCodeManager
	instanceVariableNames:''
	classVariableNames:'DefaultManager CachingSources CacheDirectoryName UseWorkTree
		WorkTreeDirectoryName'
	poolDictionaries:''
	category:'System-Support'
!

!AbstractSourceCodeManager class methodsFor:'documentation'!

copyright
"
 COPYRIGHT (c) 1995 by Claus Gittinger
	      All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"
!

documentation
"
    Abstract superclass for sourceCodeManagers.
    Concrete subclasses provide access to a source repository.
    All protocol here traps into subclassResponsbility errors.
    Please read more documentation in concrete subclasses 
    (especially: CVSSourceCodeManager) for how to use this manager.
"
! !

!AbstractSourceCodeManager class methodsFor:'initialization'!

initCacheDirPath
    |path|

    path := OperatingSystem getEnvironment:'STX_TMPPATH'.
    path isNil ifTrue:[
        path := OperatingSystem getEnvironment:'TMPPATH'.
        path isNil ifTrue:[
            path := '/tmp'
        ]
    ].
    CacheDirectoryName := path , '/stx_sourceCache'.

    "Modified: 16.12.1995 / 15:39:42 / cg"
!

initialize
    CachingSources := UseWorkTree := false.
    CacheDirectoryName := WorkTreeDirectoryName := ''.

    "Modified: 16.12.1995 / 15:39:42 / cg"
    "Created: 16.12.1995 / 15:41:00 / cg"
! !

!AbstractSourceCodeManager class methodsFor:'accessing'!

cacheDirectoryName
    ^ CacheDirectoryName
!

cacheDirectoryName:aStringOrFilename
    CacheDirectoryName := aStringOrFilename

    "Created: 16.12.1995 / 15:18:43 / cg"
!

cachingSources
    ^ CachingSources

    "Created: 16.12.1995 / 15:17:50 / cg"
    "Modified: 16.12.1995 / 15:18:03 / cg"
!

cachingSources:aBoolean
    CachingSources := aBoolean

    "Created: 16.12.1995 / 15:18:13 / cg"
!

defaultManager
    ^ DefaultManager

    "Created: 7.12.1995 / 17:14:22 / cg"
!

useWorkTree
    ^ UseWorkTree

    "Created: 16.12.1995 / 15:36:48 / cg"
    "Modified: 16.12.1995 / 15:37:17 / cg"
!

useWorkTree:aBoolean
    UseWorkTree := aBoolean

    "Modified: 16.12.1995 / 15:18:03 / cg"
    "Created: 16.12.1995 / 15:37:29 / cg"
!

workTreeDirectoryName
    ^ WorkTreeDirectoryName

    "Created: 16.12.1995 / 15:35:21 / cg"
!

workTreeDirectoryName:aStringOrFilename
    WorkTreeDirectoryName := aStringOrFilename

    "Created: 16.12.1995 / 15:35:34 / cg"
! !

!AbstractSourceCodeManager class methodsFor:'private'!

directoryFromContainerPath:containerPath
    "given a full path as in an RCS header, extract the directory (i.e. package)."

    |path idx|

    path := self pathInRepositoryFrom:containerPath.
    path notNil ifTrue:[
        idx := path indexOf:(Filename separator).
        idx ~~ 0 ifTrue:[
            path := path copyFrom:(idx + 1)
        ].
        ^ path asFilename directoryName
    ].
    ^ nil

    "
     SourceCodeManager directoryFromContainerPath:'/files/CVS/stx/libbasic/Array.st'
    "

    "Created: 25.11.1995 / 18:42:20 / cg"
    "Modified: 13.12.1995 / 13:06:15 / cg"
!

filenameFromContainerPath:containerPath
    "given some path as present in an RCS Header string, extract the containers
     name.
     OBSOLETE - this is no longer used."

    |top rest|

    containerPath notNil ifTrue:[
        top := self repositoryTopDirectory.
        top notNil ifTrue:[
            (containerPath startsWith:(top , '/')) ifTrue:[
                rest := containerPath copyFrom:(top size + 2).
                ^ rest asFilename baseName
            ]
        ]
    ].

    "Created: 25.11.1995 / 18:42:34 / cg"
    "Modified: 13.12.1995 / 13:05:14 / cg"
!

moduleFromContainerPath:containerPath
    "given a full path as in an RCS header, extract the module."

    |path idx|

    path := self pathInRepositoryFrom:containerPath.
    path notNil ifTrue:[
        idx := path indexOf:(Filename separator).
        idx == 0 ifTrue:[^ path].
        ^ path copyTo:(idx - 1)
    ].
    ^ nil

    "
     SourceCodeManager moduleFromContainerPath:'/files/CVS/stx/libbasic/Array.st'
    "

    "Created: 25.11.1995 / 18:42:20 / cg"
    "Modified: 13.12.1995 / 13:06:03 / cg"
!

pathInRepositoryFrom:containerPath
    "this tries to extract the path within a repository, given some path
     as present in an RCS Header string.
     Typically, this ought to be that string directly; 
     however, if the repository directory is accessed via a symbolic link during
     ci/co, some systems extract different strings with co.
     One such system here had a symbolic link from /phys/ibm/CVS... to /file/CVS,
     and extracted sources had /phys/ibm/CVS in their header.
     Do not depend on the code below to work correctly all the time."

    |top rest lastTop idx|

    containerPath notNil ifTrue:[
        top := self repositoryTopDirectory.
        top notNil ifTrue:[
            (containerPath startsWith:(top , '/')) ifTrue:[
                ^ containerPath copyFrom:(top size + 2).
            ].

            "/ hardcase - the repository-filename in the versionInfo
            "/ does no match my repository top.
            "/ check for mangled prefix (happens with symbolic links)

            lastTop := '/' , top asFilename baseName, '/'.
            idx := containerPath indexOfSubCollection:lastTop.
            idx ~~ 0 ifTrue:[
                ('SOURCEMGR: warning: repository path mismatch: ' , (containerPath copyTo:idx-1) , lastTop , ' vs. ' , top , '/') infoPrintNL.
                'SOURCEMGR: warning: assuming that mismatch is ok.' infoPrintNL.
                ^ containerPath copyFrom:(idx + lastTop size).
            ]
        ]
    ].
    ^ rest

    "
     SourceCodeManager pathInRepositoryFrom:'/files/CVS/stx/libbasic/Array.st'
     SourceCodeManager pathInRepositoryFrom:'/phys/ibm/CVS/stx/libbasic/Array.st'
    "

    "Created: 25.11.1995 / 18:42:20 / cg"
    "Modified: 13.12.1995 / 13:03:53 / cg"
!

repositoryTopDirectory
    ^ nil

    "Created: 25.11.1995 / 18:38:59 / cg"
!

revisionAfter:aRevisionString
    |idx s|

    idx := aRevisionString lastIndexOf:$..
    idx == 0 ifTrue:[
	^ ((Integer readFrom:aRevisionString) + 1) printString
    ].

    ^ (aRevisionString copyTo:idx) , ((Integer readFrom:(aRevisionString copyFrom:(idx+1)))+1) printString

    "
     SourceCodeManager revisionAfter:'1.2.3.4' 
     SourceCodeManager revisionAfter:'123'  
     SourceCodeManager revisionAfter:'1.24'  
    "

    "Created: 20.11.1995 / 12:54:05 / cg"
!

sourceCacheDirectory
    |dir nm|

    (nm := self cacheDirectoryName) isNil ifTrue:[^ nil].

    (dir := nm asFilename) exists ifFalse:[
        dir makeDirectory.
        dir exists ifFalse:[
            'SOURCEMGR: could not create cache dir ''' , CacheDirectoryName , '''' infoPrintNL.
            ^ nil
        ].
        "/
        "/ make it read/writable for everyone
        "/
        dir makeReadableForAll.
        dir makeWritableForAll.
        dir makeExecutableForAll.
    ].
    ^ dir

    "Modified: 9.1.1996 / 15:33:27 / cg"
!

sourceInfoOfClass:aClass
    "helper: return a classes sourceCodeInfo"

    |cls sourceInfo revInfo actualSourceFileName classFileName
     newInfo revisionInfo container|

    cls := aClass.
    cls isMeta ifTrue:[
        cls := cls soleInstance
    ].

    "/
    "/ the info given by the classes source ...
    "/
    revInfo := aClass revisionInfo.

    "/
    "/ the info given by the classes binary ...
    "/ if present, we better trust that one.
    "/
    sourceInfo := cls packageSourceCodeInfo.
    sourceInfo isNil ifTrue:[
        revInfo isNil ifTrue:[
            'SOURCEMGR: class has neither source nor compiled-in info' infoPrintNL.
            ^ nil
        ].

        container := revInfo at:#repositoryPathName ifAbsent:nil.
        container notNil ifTrue:[
            sourceInfo := revInfo.
            sourceInfo at:#directory put:(self directoryFromContainerPath:container).
            sourceInfo at:#module put:(self moduleFromContainerPath:container).
        ]
    ].


    "/
    "/ the filename I would expect from its name ...
    "/
    classFileName := Smalltalk fileNameForClass:aClass.

    newInfo := IdentityDictionary new.
    sourceInfo keysAndValuesDo:[:key :value |
        newInfo at:key put:value
    ].

    "/ check ..
    revInfo notNil ifTrue:[
        actualSourceFileName := revInfo at:#fileName ifAbsent:nil.
        actualSourceFileName notNil ifTrue:[
            actualSourceFileName ~= (classFileName , '.st') ifTrue:[
                ('SOURCEMGR: source of class ' , aClass name , ' found in ' , actualSourceFileName , ' (expected: ' , classFileName , '.st); renamed ?') infoPrintNL.
                newInfo at:#expectedFileName put:(classFileName).
                newInfo at:#renamed put:true.
                classFileName := actualSourceFileName copyWithoutLast:3
            ]
        ]
    ].

    newInfo at:#classFileName put:classFileName.
    ^ newInfo

    "Created: 25.11.1995 / 12:40:19 / cg"
    "Modified: 17.12.1995 / 16:03:31 / cg"
!

streamForClass:cls fileName:classFileName revision:revision directory:packageDir module:moduleDir cache:cacheIt
    self subclassResponsibility.
    ^ nil

    "Created: 4.11.1995 / 19:09:12 / cg"
    "Modified: 4.11.1995 / 19:15:43 / cg"
! !

!AbstractSourceCodeManager class methodsFor:'source code access'!

checkinClass:aClass fileName:classFileName directory:packageDir module:moduleDir logMessage:logMessage
    "checkin of a class into the source repository.
     Return true if ok, false if not."

    |tempDir tempFile ok|

    tempDir := (Filename newTemporaryIn:nil) makeDirectory.
    [
        |aStream|

        tempFile := tempDir construct:classFileName.
        aStream := tempFile writeStream.
        aStream isNil ifTrue:[
            'SOURCEMGR: temporary fileout failed' errorPrintNL.
            ^ false
        ].

        Class fileOutErrorSignal handle:[:ex |
            'SOURCEMGR: fileout failed' errorPrintNL.
            aStream close.
            ^ false
        ] do:[
            aClass fileOutOn:aStream withTimeStamp:false.
        ].
        aStream close.

        tempFile exists ifFalse:[
            'SOURCEMGR: temporary fileout failed' errorPrintNL.
            ^ false
        ].

        ok := self 
            checkinClass:aClass
            fileName:classFileName 
            directory:packageDir 
            module:moduleDir
            source:(tempFile name)
            logMessage:logMessage.

        ^ ok
    ] valueNowOrOnUnwindDo:[
        tempDir recursiveRemove
    ].
    ^ false

    "
     SourceCodeManager checkinClass:Array
    "

    "Created: 6.11.1995 / 18:56:00 / cg"
    "Modified: 9.12.1995 / 21:43:35 / cg"
!

checkinClass:aClass logMessage:logMessage
    "checkin of a class into the source repository.
     Return true if ok, false if not."

    |sourceInfo packageDir moduleDir classFileName|

    sourceInfo := self sourceInfoOfClass:aClass.
    sourceInfo isNil ifTrue:[
        ('SOURCEMGR: no sourceInfo for class: ' , aClass name) errorPrintNL.
        ^ false
    ].

    packageDir := sourceInfo at:#directory.
    moduleDir := sourceInfo at:#module.  "/ use the modules name as CVS module
    classFileName := sourceInfo at:#classFileName.
"/    classFileName := Smalltalk fileNameForClass:aClass.

    ^ self checkinClass:aClass fileName:(classFileName , '.st') directory:packageDir module:moduleDir logMessage:logMessage


    "
     SourceCodeManager checkinClass:Array
    "

    "Created: 6.11.1995 / 18:56:00 / cg"
    "Modified: 9.12.1995 / 21:40:44 / cg"
!

mostRecentSourceStreamForClassNamed:aClassName
    |cls sourceInfo classFileName packageDir moduleDir|

    cls := Smalltalk classNamed:aClassName.
    cls notNil ifTrue:[
        sourceInfo := self sourceInfoOfClass:cls.
    ].
    sourceInfo notNil ifTrue:[
        packageDir := sourceInfo at:#directory.
        moduleDir := sourceInfo at:#module.  "/ use the modules name as CVS module
        classFileName := sourceInfo at:#classFileName.
    ] ifFalse:[
        classFileName := Smalltalk fileNameForClass:aClassName.
        packageDir := Smalltalk sourceDirectoryNameOfClass:aClassName.
        moduleDir := 'stx'.
    ].

    packageDir isNil ifTrue:[
        'SRCMGR: could not extract packageDir' errorPrintNL.
        ^ nil
    ].

    ^ self 
        streamForClass:nil
        fileName:classFileName 
        revision:#newest 
        directory:packageDir 
        module:moduleDir
        cache:false

    "Created: 6.11.1995 / 16:09:06 / cg"
    "Modified: 10.12.1995 / 01:09:06 / cg"
!

sourceStreamFor:aClass
    "extract a classes source code and return an open readStream on it.
     The classes source code is extracted using the revision and the sourceCodeInfo,
     which itself is extracted from the classes packageString."

    ^ self sourceStreamFor:aClass revision:nil
!

sourceStreamFor:aClass revision:aRevisionStringOrNil
    "extract a classes source code and return an open readStream on it.
     A revision of nil selects the current (in image) revision.
     The classes source code is extracted using the revision and the sourceCodeInfo,
     which itself is extracted from the classes packageString."

    |classFileName revision 
     packageDir moduleDir sourceInfo revInfo|

    aRevisionStringOrNil isNil ifTrue:[
	revision := aClass binaryRevision.
	revision isNil ifTrue:[ 
	    'SOURCEMGR: class ' , aClass name , ' has no revision string' infoPrintNL.
	    ^ nil.
	]
    ] ifFalse:[
	revision := aRevisionStringOrNil
    ].

    sourceInfo := self sourceInfoOfClass:aClass.
    sourceInfo isNil ifTrue:[^ nil].

    packageDir := sourceInfo at:#directory.
    moduleDir := sourceInfo at:#module.  "/ use the modules name as CVS module
    classFileName := sourceInfo at:#classFileName.


    ^ self 
	streamForClass:aClass
	fileName:classFileName 
	revision:revision 
	directory:packageDir 
	module:moduleDir
	cache:true

    "Modified: 25.11.1995 / 17:04:38 / cg"
! !

!AbstractSourceCodeManager class methodsFor:'source code administration'!

checkForExistingContainerInModule:moduleName package:dirName container:fileName
    self subclassResponsibility.
    ^ false

    "Created: 9.12.1995 / 19:02:23 / cg"
    "Modified: 9.12.1995 / 19:43:39 / cg"
!

checkForExistingModule:moduleName
    self subclassResponsibility.
    ^ false

    "Created: 9.12.1995 / 19:02:23 / cg"
    "Modified: 9.12.1995 / 19:43:54 / cg"
!

checkForExistingModule:moduleDir package:packageDir
    self subclassResponsibility.
    ^ false

    "Created: 9.12.1995 / 19:02:23 / cg"
    "Modified: 9.12.1995 / 19:44:04 / cg"
!

createContainerFor:aClass inModule:moduleName directory:dirName container:fileName
    self subclassResponsibility.
    ^ false

    "Created: 9.12.1995 / 19:02:47 / cg"
    "Modified: 9.12.1995 / 19:28:15 / cg"
!

createModule:moduleName
    self subclassResponsibility.
    ^ false

    "Created: 9.12.1995 / 19:02:23 / cg"
    "Modified: 9.12.1995 / 19:53:23 / cg"
!

createModule:module package:package
    self subclassResponsibility.
    ^ false

    "Created: 9.12.1995 / 19:02:23 / cg"
    "Modified: 9.12.1995 / 19:54:26 / cg"
!

revisionLogOf:aClass
    "return info about the repository container and
     the revisionlog as a collection of revision entries.
     Return nil on failure.
     The returned information is a structure (IdentityDictionary)
     filled with:
	    #container          -> the RCS container file name
	    #filename           -> the actual source file name
	    #newestRevision     -> the revisionString of the newest revision
	    #numberOfRevisions  -> the number of revisions in the container
	    #revisions          -> collection of per-revision info

	    per revision info consists of one record per revision:

	      #revision              -> the revision string
	      #author                -> who checked that revision into the repository
	      #date                  -> when was it checked in
	      #state                 -> the RCS state
	      #numberOfChangedLines  -> the number of changed line w.r.t the previous

	    revisions are ordered newest first 
	    (i.e. the last entry is for the initial revision; the first for the most recent one)
	"

    ^ self
	revisionLogOf:aClass fromRevision:nil toRevision:nil

    "
     SourceCodeManager revisionLogOf:Array 
    "

    "Created: 25.11.1995 / 11:25:02 / cg"
    "Modified: 25.11.1995 / 11:56:16 / cg"
!

revisionLogOf:aClass fromRevision:rev1
    "return info about the repository container and
     (part of) the revisionlog as a collection of revision entries.
     Return nil on failure.

     The returned information is a structure (IdentityDictionary)
     filled with:
	    #container          -> the RCS container file name 
	    #filename           -> the actual source file name
	    #newestRevision     -> the revisionString of the newest revision
	    #numberOfRevisions  -> the number of revisions in the container
	    #revisions          -> collection of per-revision info (see below)

	 for some classes, additional info is returned:

	    #renamed            -> true if the class has been renamed or copied
				   and the sourceInfo is from the previous one
	    #expectedFileName   -> the filename we would expect (i.e. for the new class)

	    rev1 / rev2 specify from which revisions a logEntry is wanted:
	      If rev1 is nil, the first revision is the initial revision
	      otherwise, the log starts with that revision.
	      If rev2 is nil, the last revision is the newest revision
	      otherwise, the log ends with that revision.
	      If both are nil, no logEntries are extracted (i.e. only the header).

	    per revision info consists of one record per revision:

	      #revision              -> the revision string
	      #author                -> who checked that revision into the repository
	      #date                  -> when was it checked in
	      #state                 -> the RCS state
	      #numberOfChangedLines  -> the number of changed line w.r.t the previous

	    revisions are ordered newest first 
	    (i.e. the last entry is for the initial revision; the first for the most recent one)
	"

    ^ self revisionLogOf:aClass fromRevision:rev1 toRevision:nil

    "
     SourceCodeManager revisionLogOf:Array fromRevision:'1.40'
    "

    "Created: 6.11.1995 / 18:56:00 / cg"
    "Modified: 7.12.1995 / 11:51:15 / cg"
!

revisionLogOf:aClass fromRevision:rev1 toRevision:rev2
    "return info about the repository container and
     (part of) the revisionlog as a collection of revision entries.
     Return nil on failure.

     The returned information is a structure (IdentityDictionary)
     filled with:
            #container          -> the RCS container file name 
            #filename           -> the actual source file name
            #newestRevision     -> the revisionString of the newest revision
            #numberOfRevisions  -> the number of revisions in the container
            #revisions          -> collection of per-revision info (see below)

         for some classes, additional info is returned:

            #renamed            -> true if the class has been renamed or copied
                                   and the sourceInfo is from the previous one
            #expectedFileName   -> the filename we would expect (i.e. for the new class)

            rev1 / rev2 specify from which revisions a logEntry is wanted:
              If rev1 is nil, the first revision is the initial revision
              otherwise, the log starts with that revision.
              If rev2 is nil, the last revision is the newest revision
              otherwise, the log ends with that revision.
              If both are nil, no logEntries are extracted (i.e. only the header).

            per revision info consists of one record per revision:

              #revision              -> the revision string
              #author                -> who checked that revision into the repository
              #date                  -> when was it checked in
              #state                 -> the RCS state
              #numberOfChangedLines  -> the number of changed line w.r.t the previous

            revisions are ordered newest first 
            (i.e. the last entry is for the initial revision; the first for the most recent one)
        "

    |cls sourceInfo packageDir moduleDir classFileName info|

    sourceInfo := self sourceInfoOfClass:aClass.
    sourceInfo isNil ifTrue:[^ nil].

    packageDir := sourceInfo at:#directory.
    moduleDir := sourceInfo at:#module.  "/ use the modules name as CVS module
"/    classFileName := Smalltalk fileNameForClass:aClass.
    classFileName := sourceInfo at:#classFileName.

    info := self 
        revisionLogOf:aClass
        fromRevision:rev1 
        toRevision:rev2
        fileName:(classFileName , '.st')
        directory:packageDir 
        module:moduleDir.

    info notNil ifTrue:[
        (sourceInfo includesKey:#renamed) ifTrue:[
            info at:#renamed put:(sourceInfo at:#renamed)
        ].
        (sourceInfo includesKey:#expectedFileName) ifTrue:[
            info at:#expectedFileName put:(sourceInfo at:#expectedFileName)
        ]
    ].
    ^ info

    "
     SourceCodeManager revisionLogOf:Array fromRevision:'1.40' toRevision:'1.43' 
     SourceCodeManager revisionLogOf:XtBoxNew 
    "

    "Created: 6.11.1995 / 18:56:00 / cg"
    "Modified: 9.12.1995 / 22:04:08 / cg"
!

revisionLogOf:cls fromRevision:rev1 toRevision:rev2 fileName:classFileName directory:packageDir module:moduleDir 
    "this must be implemented by a concrete source-code manager"

    self subclassResponsibility.

    "Created: 15.11.1995 / 18:12:51 / cg"
    "Modified: 25.11.1995 / 11:56:38 / cg"
!

writeHistoryLogSince:timeGoal filterSTSources:filter to:aStream
    self writeHistoryLogSince:timeGoal filterSTSources:filter filterUser:nil to:aStream

    "Modified: 13.12.1995 / 11:03:01 / cg"
!

writeHistoryLogSince:timeGoal to:aStream
    self writeHistoryLogSince:timeGoal filterSTSources:true filterUser:nil to:aStream

    "Created: 13.12.1995 / 10:28:27 / cg"
    "Modified: 13.12.1995 / 11:00:49 / cg"
!

writeRevisionLogMessagesFrom:log to:aStream
    "helper; send the revisionlog to aStream"

    self writeRevisionLogMessagesFrom:log withHeader:true to:aStream

    "Created: 10.12.1995 / 16:51:30 / cg"
!

writeRevisionLogMessagesFrom:log withHeader:header to:aStream
    "helper; send the revisionlog to aStream"

    header ifTrue:[
        (log at:#renamed ifAbsent:false) ifTrue:[
            aStream nextPutAll:'  Class was probably renamed; revision info is from original class.'.
            aStream cr; nextPutAll:'  You may have to create a new container for it.'.
            aStream cr; cr.
        ].

        aStream nextPutAll:'  Total revisions: '; nextPutAll:(log at:#numberOfRevisions) printString; cr.
        aStream nextPutAll:'  Newest revision: '; nextPutAll:(log at:#newestRevision) printString; cr.
    ].

    (log at:#revisions) do:[:entry |
        |logMsg|

        aStream cr.
        aStream nextPutAll:'  revision '; nextPutAll:(entry at:#revision); tab.
        aStream nextPutAll:' date: '; nextPutAll:(entry at:#date); tab.
        aStream nextPutAll:' author: '; nextPutAll:(entry at:#author); tab.
        aStream cr.

        logMsg := entry at:#logMessage.
        (logMsg isEmpty or:[logMsg isBlank or:[logMsg withoutSeparators = '.']]) ifTrue:[
            logMsg := '*** empty log message ***'
        ].
        aStream tab; nextPutAll:logMsg.
        aStream cr.
    ].

    "Created: 16.11.1995 / 13:25:30 / cg"
    "Modified: 10.12.1995 / 16:51:15 / cg"
!

writeRevisionLogOf:aClass fromRevision:rev1 toRevision:rev2 to:aStream
    "extract a classes log and append it to aStream."

    |log |

    log := self revisionLogOf:aClass fromRevision:rev1 toRevision:rev2.
    log isNil ifTrue:[^ false].

    self writeRevisionLogMessagesFrom:log to:aStream.
    ^ true

    "
     SourceCodeManager writeRevisionLogOf:Array fromRevision:'1.40' toRevision:'1.43' to:Transcript 
    "

    "Created: 6.11.1995 / 18:56:00 / cg"
    "Modified: 25.11.1995 / 17:24:55 / cg"
!

writeRevisionLogOf:aClass to:aStream
    "extract a classes log and append it to aStream."

    ^ self
	writeRevisionLogOf:aClass fromRevision:nil toRevision:nil to:aStream

    "
     SourceCodeManager writeRevisionLogOf:Array to:Transcript 
    "
! !

!AbstractSourceCodeManager class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/libbasic3/Attic/AbstrSCMgr.st,v 1.40 1996-01-09 14:33:49 cg Exp $'
! !
AbstractSourceCodeManager initialize!
