"
 COPYRIGHT (c) 1995 by Claus Gittinger
	      All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"

AbstractSourceCodeManager subclass:#CVSSourceCodeManager
	instanceVariableNames:''
	classVariableNames:'Verbose CVSRoot CVSWorkDirectory'
	poolDictionaries:''
	category:'System-Support'
!

!CVSSourceCodeManager class methodsFor:'documentation'!

copyright
"
 COPYRIGHT (c) 1995 by Claus Gittinger
	      All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"
!

documentation
"
    SourceCodeManager which accesses the sourcecode through cvs.
    It requires the shell environment variable CVSROOT to be set
    to the top of the repository tree. Under that top, for each module
    (especially the stx module itself), a subdirectory containing that modules
    directories must exist.

    If not specified otherwise, the module defaults to 'stx'. 
    If a class has its home in another module, it has to be compiled
    with a corresponding entry in the package string (see below).
    Within a module, classes are organized in classlibraries, and the corresponding
    source code is found in various subdirectories of CVSROOT. The directories
    name can also be provided by the package string - if not, it defaults to the
    libraries name.
    The package string controls all this, and is REQUIRED in order for the sourceCodeManager
    to be able to retrieve a classes source.
    The package strings format is:
        '...any infotext....(sourceCodeInfo)'
    the sourceCOdeInfo consists of multiple entries, separated by colons.
    The following formats are allowed:
        (libraryName)                   - module defaults to 'stx'; directory to the name of the library
        (foo/bar/x)                     - module defaults to 'stx'; dir is 'foo/bar/x'; library name defaults to x
        (module:directory:libname)      - specifies all components
    The first formats are intermediate, for backward compatibility. We urge everyone
    to use the last format (by changing the Make.proto files and add a -P flag to
    the stc compiler flags (see libbasic/Make.proto as an example).

    Notice, that the sourceCodeManager is (currently) only consulted, if no
    source subdirectory exists, or no source file is found there. If it does exist,
    that one is supposed to contain an up-to-date version of the classes source
    (this is temporary - in the future the probe order will be reversed, trying
     the source repository first).
    The source directory is required for systems which do NO source code control.

    Setup:
        - make certain, that the commands 'cvs' and 'co' are available on your system.
        - add setup for the CVSROOT environment variable to your .bashrc / .login / .profile
        - Make certain that either no local source directory exists, or it is empty or it contains
          only source code for files NOT found in the repository (this will not be required in future versions).

    Caveat:
        currently, the rcs container is accessed directly, since cvs mangles the files ident-string and
        thus does not correctly recreate the original file (all source is offset by some characters w.r.t. the
        compiled file).
        Therefore, sevrer-mode CVS is not supported.
"
! !

!CVSSourceCodeManager class methodsFor:'initialization'!

initialize
    |s f top|

    "/
    "/ if CVSROOT is non-empty and both cvs and co are available
    "/ as commands, assume this system uses a cvs source code management.
    "/
    (CVSRoot := OperatingSystem getEnvironment:'CVSROOT') notNil ifTrue:[
	"/
	"/ check if there is an stx directory there
	"/
	((f := CVSRoot asFilename) exists 
	and:[f isDirectory
	and:[(top := f construct:'stx') exists
	and:[top isDirectory
	and:[top isReadable]]]]) ifTrue:[
            (OperatingSystem canExecuteCommand:'cvs') ifTrue:[
                self initCacheDirPath.
                CachingSources := true.
                Smalltalk at:#SourceCodeManager put:self.
                DefaultManager := self.    
"/                    'CVS SourceCodeManager installed.' infoPrintNL.
                "/
                "/ optionally set the WorkTreeDirectoryName from $STX_WORKTREE;
                "/ if non-nil, a working tree is kept there
                "/ and updated/commited files are not removed.
                "/ If you use a regular (make-) tree, 
                "/ set WorkTreeDirectoryName (or the environment variable) to that.
                "/
                "/ this is not yet finished.
                "/
                s := OperatingSystem getEnvironment:'STX_WORKTREE'.
                s notNil ifTrue:[
                    WorkTreeDirectoryName := s.
                    UseWorkTree := true.
                ].
	    ]
        ]
    ]

    "
     AbstractSourceCodeManager initialize
     CVSSourceCodeManager initialize
    "

    "Created: 4.11.1995 / 19:14:38 / cg"
    "Modified: 16.12.1995 / 15:47:05 / cg"
    "Modified: 19.12.1995 / 14:25:46 / stefan"
! !

!CVSSourceCodeManager class methodsFor:'private'!

checkinTroubleDialog:title message:message log:log abortable:abortable option:optionTitle
    |l box logStrings list listView optionPressed|

    l := log collect:[:line | line withTabsExpanded].
    list := SelectionInList with:l.

    box := Dialog new.
    box label:(title).

    (box addTextLabel:message) borderWidth:0.

    listView := SelectionInListView on:list.
    listView disable.
    listView height:200.
    box addComponent:(HVScrollableView forView:listView miniScrollerH:true) tabable:true.
    box addVerticalSpace.

    abortable ifTrue:[
        box addAbortButton
    ].
    optionTitle notNil ifTrue:[
        box addOkButton:(Button label:optionTitle action:[optionPressed := true. box hide]).
    ].
    box addOkButton.

    box extent:(box preferredExtent).
    box minExtent:box extent.
    box maxExtent:box extent.

    box showAtPointer.

    box destroy.
    optionPressed == true ifTrue:[^ #option].
    ^ box accepted

    "Created: 10.12.1995 / 17:34:33 / cg"
!

createEntryFor:pathInRepository in:aDirectory  revision:rev date:date special:special overwrite:overwrite
    "create an CVS/Repository and CVS/Entries entry for filename. 
     If the path does not exist, it will be created
     If overwrite is true, an existing Entry will be overwritten.
    "

    |filename directory cvsDir name entries firstPart s|

    filename := pathInRepository asFilename.
    cvsDir := (aDirectory asFilename construct:'CVS') recursiveMakeDirectory.
    entries := cvsDir construct:'Entries'.
    name := filename baseName.
    firstPart := '/', name, '/'.

    s := entries readStream.
    s isNil ifTrue:[
        "/
        "/ no Repository yet, create one
        "/
        |index root|

        index := CVSRoot indexOf:$:.
        index > 0 ifTrue:[
            root := CVSRoot copyFrom:(index+1).
        ] ifFalse:[
            root := CVSRoot.
        ].
        s := (cvsDir construct:'Repository') writeStream.
        s nextPutLine:(root, '/', filename directory asString).
        s close.

        "/ make new Entries file.
        s := entries writeStream.
    ] ifFalse:[
        |newName newStream line|

        newName := Filename newTemporaryIn:cvsDir.
        newStream := newName writeStream.
        [(line := s nextLine) notNil] whileTrue:[
            (line findString:firstPart) ~= 1 ifTrue:[
                "/ we have found an entry for ourself.
                "/ If we do not want to overwrite it, clean up everything and return.
                overwrite ifFalse:[
                    newName remove.
                    newStream close.
                    s close.
                    ^ self
                ].
                newStream nextPutLine:line
            ].
        ].
        newName renameTo:entries.
        s close.
        s := newStream.
    ].
    s nextPutLine:firstPart, rev, '/', date, '/', special, '/'.
    s close.

    "
        self createEntryFor:'foo/xxx/yyy' in:'foo/xxx' revision:'1.22' date:'dummy' special:''
        self createEntryFor:'foo/xxx/yyy' in:'foo/xxx' revision:'3.44' date:'new dummy' special:'special'
    "

    "Modified: 19.12.1995 / 22:02:56 / stefan"
    "Created: 20.12.1995 / 11:09:54 / stefan"
!

createTempDirectory:packageDir forModule:moduleDir
    |tempdir dir|

    tempdir := Filename newTemporaryIn:nil.
    tempdir exists ifTrue:[
        tempdir recursiveRemove.
    ].
    tempdir makeDirectory.

    moduleDir notNil ifTrue:[
        dir := tempdir construct:moduleDir.
        dir makeDirectory.
        packageDir notNil ifTrue:[
            dir := dir construct:packageDir.
            dir recursiveMakeDirectory.
        ].
        dir exists ifFalse:[
            tempdir recursiveRemove.
            'CVSMGR: cannot create temporary directory' infoPrintNL.
            ^ nil.
        ].
    ].
    ^ tempdir

    "self createTempDirectory:'fooPackage' forModule:'fooModule'"

    "Created: 9.12.1995 / 19:14:35 / cg"
    "Modified: 11.12.1995 / 13:26:13 / cg"
    "Modified: 19.12.1995 / 16:13:02 / stefan"
!

cvsTimeString:absoluteTime
    "convert an absoluteTime to a string suitable for the CVS/Entries file:

                         123456789012345678901234
                         Tue Dec 19 20:56:26 1995
    "

    |date time stream|

    date := absoluteTime asDate.
    time := absoluteTime asTime.
    stream := WriteStream on:(String new:24).

    (#('Mon' 'Tue' 'Wed' 'Thu' 'Fri' 'Sat' 'Sun') at:(date dayInWeek)) printOn:stream.
    stream space.

    (#('Jan' 'Feb' 'Mar' 'Apr' 'May' 'Jun' 'Jul' 'Aug' 'Sep' 'Oct' 'Nov' 'Dec') 
        at:(date month)) printOn:stream.
    stream space.

    date day printOn:stream.
    stream space.

    time print24HourFormatOn: stream.
    stream space.

    date year printOn:stream.

    ^ stream contents


    "
     CVSSourceCodeManager cvsTimeString:(AbsoluteTime now)
    "

    "Modified: 20.12.1995 / 11:03:32 / stefan"
!

initialRevisionStringFor:aClass inModule:moduleDir package:packageDir container:fileName
    "/ do not make the string below into one string;
    "/ RCS would expand it into a wrong rev-string

    ^ '$' , 'Header: ' , CVSRoot , '/' , moduleDir , '/' , packageDir , '/' , fileName , ',v $'

    "Created: 9.12.1995 / 21:28:11 / cg"
    "Modified: 12.12.1995 / 19:41:11 / cg"
!

releaseAndRemove:tempdir outputTo:outputFilename
    |cmd redirect|

    outputFilename isNil ifTrue:[
        redirect := '/dev/null'
    ] ifFalse:[
        redirect := outputFilename name
    ].

    "/
    "/ release it towards cvs
    "/
    cmd := 'cd ' , tempdir name , ';cvs -l release > ' , redirect.
    Verbose == true ifTrue:[
        ('CVSMGR: executing: ' , cmd) infoPrintNL.
    ].

    (OperatingSystem executeCommand:cmd) ifFalse:[
"/        'CVSMGR: failed to release: ' errorPrint. cmd errorPrintNL.
    ].

    tempdir recursiveRemove.

    "Created: 10.12.1995 / 12:56:15 / cg"
!

repositoryTopDirectory
    ^ CVSRoot

    "
     CVSSourceCodeManager repositoryTopDirectory
    "

    "Created: 25.11.1995 / 18:39:19 / cg"
    "Modified: 9.12.1995 / 19:24:25 / cg"
!

revisionStringFromLog:log entry:entry
    ^ '$Header: ' , (log at:#container) ,
      ' ' , (entry at:#revision) ,
      ' ' , (entry at:#date) , 
      ' ' , (entry at:#author) , 
      ' ' , (entry at:#state) ,
      ' $'

    "Created: 7.12.1995 / 20:41:25 / cg"
!

updatedRevisionStringOf:aClass forRevision:newRevision with:originalVersionString
    |idx leftPart rightPart vsnString newVsn|

    "/ search for ,v
    idx := originalVersionString indexOfSubCollection:'.st,v'.
    idx == 0 ifTrue:[^ nil].
    leftPart := originalVersionString copyTo:(idx - 1 + 5).
    rightPart := (originalVersionString copyFrom:(idx + 5)) withoutSpaces.
    idx := rightPart indexOfSeparator.
    idx == 0 ifTrue:[^ nil].
    vsnString := rightPart copyTo:idx - 1.
    rightPart := rightPart copyFrom:idx + 1.

    newRevision isNil ifTrue:[
        (vsnString endsWith:'m') ifTrue:[
             "/ alread a modified class
"/            ('already modified: ' , vsnString) printNL.
            ^ nil
        ].
        newVsn := vsnString , 'm'
    ] ifFalse:[
        newVsn := newRevision
    ].
    ^ leftPart , ' ' , newVsn , ' ' , rightPart

    "Created: 7.12.1995 / 20:23:38 / cg"
    "Modified: 13.12.1995 / 17:13:45 / cg"
! !

!CVSSourceCodeManager class methodsFor:'source code access'!

checkinClass:cls fileName:classFileName directory:packageDir module:moduleDir source:sourceFileName logMessage:logMessage
    "enter a classes source code (which has been already filed out into sourceFileName)
     into the source repository.
     Return true if ok, false if not."

    |tempdir cmd checkoutDir checkoutName logMsg revision merge 
     cmdOut whatHappened f s entry idx changeLog changesAsLogged l 
     newRevision newString binRevision className msg answer didMerge
     mySource mergedSource modulePath time|

merge := false. "/ for now ....
merge := true. "/ for now ....

    className := cls name.

    merge ifTrue:[
        revision := cls revision.
        revision isNil ifTrue:[ 
            ('CVSMGR: class ' , className, ' has no revision string') errorPrintNL.
            (self confirm:('class ' , className, ' has no (usable) revision string.\\check in as newest ?') withCRs)
            ifFalse:[
                ^ false.
            ].
        ].
        (binRevision := cls binaryRevision) notNil ifTrue:[
            revision ~= binRevision ifTrue:[
                ('CVSMGR: class ' , className , ' is based upon ' , binRevision , ' but has revision ' , revision) infoPrintNL
            ]
        ]
    ].
    revision isNil ifTrue:[
        revision := '1.0'
    ].

    logMsg := logMessage.
    (logMsg isNil or:[logMsg isEmpty]) ifTrue:[
        logMsg := 'checkin from browser'.
    ].

    cmdOut := Filename newTemporary.
    cmdOut exists ifTrue:[
        cmdOut remove.
    ].

    "/
    "/ in CVS, we have to checkout the file first, in order
    "/ to get up-to-date CVS entries, and also to be able to merge in
    "/ other users changes.
    "/

    "/
    "/ first, create a temporary work tree
    "/
    tempdir := self createTempDirectory:nil forModule:nil.
    tempdir isNil ifTrue:[^ false].

    "/
    "/ next, create CVS/Entries and CVS/Repository with version information of current version
    "/
    modulePath :=  moduleDir , '/' , packageDir. 
    checkoutName :=  modulePath , '/' , classFileName.

    "/
    "/ correct our current time, so that converting it will give us UTC
    "/
    time := AbsoluteTime now subtractSeconds:1.
    time := time addSeconds:(time utcOffset).

    self createEntryFor:checkoutName 
         in:(tempdir construct:modulePath) 
         revision:revision
         date:(self cvsTimeString:time)
         special:''
         overwrite:true.

    "/
    "/ copy-over our current version
    "/
    cmd := 'mv ' , sourceFileName , ' ' , tempdir name , '/' , checkoutName.
"/ ('move over: ' , cmd) infoPrintNL.
    (OperatingSystem executeCommand:cmd) ifFalse:[
"/        'failed: ' errorPrint. cmd errorPrintNL.
        tempdir recursiveRemove.
        'CVSMGR: cannot copy-over filedOut class source' errorPrintNL.
        ^ false.
    ].

merge ifTrue:[

    "/
    "/ synchronize i.e. merge in any changes
    "/
    self activityNotification:'merging with repository version'.

    cmd := 'cd ' , tempdir name , ';cvs -d ', CVSRoot, ' update ' , checkoutName , ' > ' , cmdOut name.
    Verbose == true ifTrue:[
        ('CVSMGR: executing: ' , cmd) infoPrintNL.
    ].

    (OperatingSystem executeCommand:cmd) ifFalse:[
        'CVSMGR: failed to execute: ' infoPrint. cmd infoPrintNL.
        tempdir recursiveRemove.
        cmdOut remove.
        'CVSMGR: cannot merge current source with repository version' errorPrintNL.
        ^ false.
    ].

    "/
    "/ check what happened - the contents of the cmdOut file may be:
    "/   empty   -> nothing changed
    "/   M xxx   -> merged-in changes from other users
    "/   C xxx   -> a conflict occured and the differences have been merged into the source
    "/              needs special action
    "/
    whatHappened := cmdOut contentsOfEntireFile asString.
    cmdOut remove.

    Verbose == true ifTrue:[
        ('CVSMGR: result is: ' , whatHappened) infoPrintNL.
    ].
    (whatHappened isNil or:[whatHappened isEmpty]) ifTrue:[
        "/
        "/ no change
        "/
        tempdir recursiveRemove.
        Transcript showCr:'no change in ' , className , ' (repository unchanged)'.
        self information:'nothing changed in ' , className.
        ^ true
    ].

    revision isNil ifTrue:[
        changeLog := self revisionLogOf:cls.
    ] ifFalse:[
        changeLog := self revisionLogOf:cls fromRevision:(self revisionAfter:revision) toRevision:nil.
    ].
    s := WriteStream on:String new.
    self writeRevisionLogMessagesFrom:changeLog withHeader:false to:s.
    changesAsLogged := s contents.
    didMerge := false.

    (whatHappened startsWith:'M ') ifTrue:[
        "/
        "/ merged in changes
        "/
        (changeLog at:#revisions) isEmpty ifTrue:[
            "/
            "/ pretty good - nothing has changed in the meanwhile
            "/
            Transcript showCr:'checking in ' , className , ' ...'
        ] ifFalse:[
            "/
            "/ someone else has changed things in the meanwhile, but there is no conflict
            "/ and version have been merged.
            "/
            didMerge := true.
            changesAsLogged := changesAsLogged asCollectionOfLines.

            s := WriteStream on:String new.
            cls fileOutOn:s withTimeStamp:false.
            mySource := s contents asString.
            mergedSource := (tempdir name , '/' , checkoutName) asFilename readStream contents asString.

            mySource = mergedSource ifTrue:[
                msg := 'The source of ' , className , ' has been changed in the meanwhile as listed below.

I have merged your version with the newest repository version, 
and found no differences between the result and your current version
(i.e. your version seemed up-to-date).'.

                self checkinTroubleDialog:'Merging versions'
                               message:msg 
                               log:changesAsLogged
                               abortable:false 
                               option:nil.
                didMerge := false.
            ] ifFalse:[
                msg := 'The source of ' , className , ' has been changed in the meanwhile as listed below.

If you continue, your new changes (based upon rev. ' , revision printString , ') will be MERGED
into the newest revision. This will combine the other version with your changes
into a new common revision which may be different from both. 
Although this is a nice feature, it may fail to create the expected result in certain situations. 

You should carefully check the result - by comparing the current version with the
most recent version in the repository. If that does not contain an acceptable version,
change methods as required and check in again. 
Be aware, that after that, the actual repository version is different from your current classes,
and you should update your class from the repository.

Continue ?'.

                answer := self checkinTroubleDialog:'Merging versions'
                               message:msg 
                               log:changesAsLogged
                               abortable:true
                               option:'stop - see first'.
                answer ~~ true ifTrue:[
                    answer == #option ifTrue:[
                        DiffTextView 
                            openOn:mySource
                            label:'current version'
                            and:mergedSource
                            label:'merged version'.
                            
                    ].
                    Transcript showCr:'checkin aborted - (no merge; repository unchanged)'.
                    tempdir recursiveRemove.
                    ^ false.
                ].
            ].

"/                changesAsLogged := (changesAsLogged asStringCollection collect:[:line | line withTabsExpanded]) asString.
"/                msg := 'The source of ' , className , ' has been changed in the meanwhile as follows:
"/' , changesAsLogged , '
"/
"/If you continue, your new changes (based upon rev. ' , revision , ') will be MERGED
"/into the newest revision. This will combine the other version with your changes
"/into a new common revision which is different from both. 
"/Although convenient, it may fail to create the expected result in certain situations. 
"/
"/You should carefully check the result - by comparing the current version with the
"/most recent version in the repository. If that does not contain an acceptable version,
"/change methods as required and check in again. Be aware, that the actual repository version
"/is different from your current classes.
"/
"/Continue ?'.
"/                (self confirm:msg) ifFalse:[
"/                    Transcript showCr:'checkin aborted - (no merge; repository unchanged)'.
"/                    tempdir recursiveRemove.
"/                    ^ false.
"/                ].
            Transcript showCr:'checking in ' , className , ' (merged other changes) ...'
        ]
    ] ifFalse:[
        (whatHappened startsWith:'C ') ifTrue:[
            "/
            "/ conflict; someone else checked in something in the meanwhile,
            "/ and there is a conflict between this version and the checked in version.
            "/

            changesAsLogged := changesAsLogged asCollectionOfLines.

            msg := 'The source of ' , className , ' has been changed in the meanwhile as listed below.

Your new changes (based upon rev. ' , revision printString , ') CONFLICT with those changes.

You should fix things by comparing your class with the most recent repository version
and change your methods avoiding conflicts. The checkin again.
'.

            (self checkinTroubleDialog:'Version conflict'
                 message:msg 
                 log:changesAsLogged
                 abortable:false
                 option:'show conflicts') == #option ifTrue:[
                "/
                "/ for now, show the conflicts in a dump view
                "/ Later, there will be a 3-way DiffTextView ...
                "/
                Diff3TextView
                    openOnMergedText:(tempdir name , '/' , checkoutName) asFilename readStream contents 
                    label:'original' label:'repository' label:'your version'. 
"/                TextView 
"/                    openWith:(tempdir name , '/' , checkoutName) asFilename readStream contents
"/                    title:'conflicts'.
            ].

"/                changesAsLogged := (changesAsLogged asCollectionOfLines asStringCollection collect:[:line | line withTabsExpanded]) asString.
"/                self warn:'The source of ' , className , ' has been changed in the meanwhile as follows:
"/' , changesAsLogged , '
"/Your new changes (based upon rev. ' , revision , ') CONFLICT with those changes - please fix.'.

            'CVSMGR: cannot (for now) checkin; conflicts found' errorPrintNL.
            Transcript showCr:'checkin of ' , className , ' aborted (conflicting changes; repository unchanged)'.
            tempdir recursiveRemove.
            ^ false.
        ] ifFalse:[
            "/
            "/ unexpected
            "/
            self warn:'unexpected message from CVS:
' , whatHappened , '

No checkin performed.'.
            Transcript showCr:'*** cannot checkin ' , className , ' (unexpected CVS response; repository unchanged)'.
            tempdir recursiveRemove.
            ^ false.
        ]
    ].
].

    "/
    "/ now check it in again
    "/
    self activityNotification:'saving in repository'.

    logMsg := logMsg replChar:$"  withString:'\"'.
    cmd := 'cd ' , tempdir name , ';cvs -d ', CVSRoot, ' commit -m "', logMsg ,'" ' , checkoutName, ' 2> ' , cmdOut name.
    Verbose == true ifTrue:[
        ('CVSMGR: executing: ' , cmd) infoPrintNL.
    ].
    (OperatingSystem executeCommand:cmd) ifFalse:[
        Verbose == true ifTrue:[
            'CVSMGR: failed to execute: ' infoPrint. cmd infoPrintNL.
        ].
        whatHappened := cmdOut contentsOfEntireFile asString.
        self warn:'The following problem was reported by cvs:

' , whatHappened , '

The class has NOT been checked into the repository.'.

       'CVSMGR: cannot checkin modified class source' errorPrintNL.
        cmdOut remove.
        tempdir recursiveRemove.
        ^ false.
    ].
    whatHappened := cmdOut contentsOfEntireFile asString.

    tempdir recursiveRemove.
    cmdOut remove.

    "/
    "/ fetch the new revision nr as found in the commit commands output
    "/

    (whatHappened isNil or:[whatHappened isEmpty]) ifTrue:[
        'CVSMGR: oops - unexpected empty checkin command output' errorPrintNL.
    ] ifFalse:[
        whatHappened := whatHappened asCollectionOfLines asStringCollection.
        idx := whatHappened indexOfLineStartingWith:'new revision:'.
        idx == 0 ifTrue:[
            'CVSMGR: oops - unexpected checkin command output (no new-revision info)' errorPrintNL.
        ] ifFalse:[
            l := whatHappened at:idx.
            newRevision := (l copyFrom:14 to:(l indexOf:$; startingAt:14)-1) withoutSpaces.
        ]
    ].

    "/
    "/ if there was no merge (i.e. the current version has been checked in unchanged):
    "/   patch the classes revisionInfo (but keep binaryRevision unchanged) to the new revision
    "/   this makes everyone here believe, that the incore version of the class is based upon
    "/   the newly checked in version.
    "/   (however, the binaryRevision must remain as it is - we will need it to fetch the sourceCode
    "/    correctly for all unchanged methodss)
    "/
    "/ if there was a merge (i.e. the repository now contains a merge of the current and some
    "/ other version):
    "/    patch the classes revisionInfo (again, keep the binaryRevision) to the old revision
    "/    and add a 'm' (for merged).
    "/    If we later checkin again, the new checkin will be again based on the current revision
    "/
    newRevision notNil ifTrue:[
        didMerge ifFalse:[
            self activityNotification:'fetch revision number'.

            changeLog := self revisionLogOf:cls fromRevision:newRevision toRevision:newRevision.
            (changeLog isNil or:[(changeLog at:#revisions) size ~~ 1]) ifTrue:[
                'CVSMGR: oops - failed to update revisionString (no log)' errorPrintNL
            ] ifFalse:[
                entry := (changeLog at:#revisions) first.
                newString := self revisionStringFromLog:changeLog entry:entry.
                cls updateVersionMethodFor:newString.
                cls revision ~= newRevision ifTrue:[
                    'CVSMGR: oops - failed to update revisionString' errorPrintNL
                ]
            ]
        ] ifTrue:[
            newString := self updatedRevisionStringOf:cls forRevision:nil with:cls revisionString.
            cls updateVersionMethodFor:newString.
        ].
    ].

    Class addChangeRecordForClassCheckIn:cls.
    ^ true

    "
     SourceCodeManager checkinClass:Array logMessage:'testing only'
    "

    "Modified: 17.12.1995 / 16:02:37 / cg"
    "Modified: 20.12.1995 / 17:34:53 / stefan"
!

streamForClass:cls fileName:classFileName revision:revision directory:packageDir module:moduleDir cache:cacheIt
    "extract a source file and return an open readStream on it,
     or nil if the extract failed."

    |dir cachedSourceFilename temporaryFile file cachedFile cmd fullName revisionArg stream
     checkoutDir checkoutName fullTempName fullCachedName tempdir cmdOut|


    "/ if not already existing, create a cache directory
    "/ where we deposit sources.
    "/ this is used as a cache for further requests, since
    "/ accessing the repository may be slow.
    "/ (if cvs uses a remote connection via ppp, for example)
    "/

    (dir := self sourceCacheDirectory) isNil ifTrue:[
        'CVSMGR: no source cache directory' infoPrintNL.
        ^ nil
    ].

    fullName := moduleDir , '/' , packageDir , '/' , classFileName , '.st'.

    (revision isNil or:[revision == #newest]) ifTrue:[
        cachedSourceFilename := classFileName.
    ] ifFalse:[
        cachedSourceFilename := classFileName , '_' , revision.
    ].

    cachedFile := (dir construct:cachedSourceFilename) asFilename.
    cachedFile exists ifTrue:[
"/        ('CVSMGR: found existing: ', cachedFile name) infoPrintNL.
        ^ cachedFile readStream
    ].

    "/
    "/ first, create a temporary work tree
    "/ Do not make module and package directories, their existence cause cvs checkout to fail in server mode
    "/
    tempdir := self createTempDirectory:nil forModule:nil.
    tempdir isNil ifTrue:[^ nil].

"/    tempdir := Filename newTemporaryIn:nil.
"/    tempdir exists ifTrue:[
"/        tempdir recursiveRemove.
"/    ].
"/    tempdir makeDirectory.
"/    dir := tempdir construct:moduleDir.
"/    dir makeDirectory.
"/    dir := dir construct:packageDir.
"/    dir recursiveMakeDirectory.
"/    dir exists ifFalse:[
"/        tempdir recursiveRemove.
"/        'CVSMGR: cannot create temporary directory' infoPrintNL.
"/        ^ nil.
"/    ].

    cmdOut := Filename newTemporary.
    cmdOut exists ifTrue:[
        cmdOut remove.
    ].

    "/
    "/ check it out there
    "/
    checkoutDir :=  moduleDir , '/' , packageDir.
    checkoutName :=  checkoutDir , '/' , classFileName , '.st'.

    (revision isNil or:[revision == #newest]) ifTrue:[
        cachedSourceFilename := classFileName.
        revisionArg := ''.
    ] ifFalse:[
        cachedSourceFilename := classFileName , '_' , revision.
        revisionArg := ' -r ' , revision.
    ].

    self activityNotification:'checking out source'.
    cmd := 'cd ' , tempdir name , ';cvs -d ', CVSRoot, ' -l checkout' , revisionArg , ' ', checkoutName , ' > ' , cmdOut name.
    Verbose == true ifTrue:[
        ('CVSMGR: executing: ' , cmd) infoPrintNL.
    ].

    (OperatingSystem executeCommand:cmd) ifFalse:[
        'CVSMGR: failed to execute: ' errorPrint. cmd errorPrintNL.
        cmdOut remove.
        tempdir recursiveRemove.
        ('CVSMGR: cannot checkout ' , checkoutName) errorPrintNL.
        ^ nil.
    ].

    cmdOut remove.
    fullTempName := tempdir name , '/' , checkoutName.
    fullCachedName := CacheDirectoryName , '/' , cachedSourceFilename.

    cacheIt ifTrue:[
        (OperatingSystem errorSignal catch:[
            fullTempName asFilename moveTo:fullCachedName
        ]) ifTrue:[
            ('CVSMGR: failed to rename ', fullTempName, ' to ', cachedSourceFilename) infoPrintNL.
            tempdir recursiveRemove.
            ^ nil
        ].
        stream := cachedFile readStream
    ] ifFalse:[
        stream := fullTempName asFilename readStream.
    ].

    self releaseAndRemove:tempdir outputTo:nil. 
    ^ stream

    "Created: 4.11.1995 / 19:46:20 / cg"
    "Modified: 17.12.1995 / 17:52:44 / cg"
    "Modified: 19.12.1995 / 16:15:26 / stefan"
! !

!CVSSourceCodeManager class methodsFor:'source code administration'!

checkForExistingContainerInModule:moduleDir package:packageDir container:fileName
    |fullName|

    fullName := moduleDir , '/' , packageDir , '/' , fileName.

    ^ OperatingSystem executeCommand:'cvs -n -d ', CVSRoot, ' checkout ', fullName.

    "CVSSourceCodeManager checkForExistingContainerInModule:'stx' package:'libbasic' container:'Integer.st'"

    "Created: 9.12.1995 / 19:13:37 / cg"
    "Modified: 19.12.1995 / 14:15:43 / stefan"
!

checkForExistingModule:moduleDir

    ^ OperatingSystem executeCommand:'cvs -n -d ', CVSRoot, ' checkout ', moduleDir

    "CVSSourceCodeManager checkForExistingModule:'stx'"

    "Created: 9.12.1995 / 19:13:37 / cg"
    "Modified: 19.12.1995 / 14:12:07 / stefan"
!

checkForExistingModule:moduleDir package:packageDir
    |fullName|

    fullName := moduleDir , '/' , packageDir.

    ^ OperatingSystem executeCommand:'cvs -n -d ', CVSRoot, ' checkout ', fullName.

    "CVSSourceCodeManager checkForExistingModule:'stx' package:'libbasic'"

    "Created: 9.12.1995 / 19:13:37 / cg"
    "Modified: 19.12.1995 / 14:18:42 / stefan"
!

createContainerFor:aClass inModule:moduleDir package:packageDir container:fileName
    |fullName tempdir checkoutName cmdOut cmd tempFile idx aStream whatHappened l newRevision
     changeLog entry newString startIdx|

    "/
    "/ first, create a temporary work tree
    "/
    tempdir := self createTempDirectory:packageDir forModule:moduleDir.
    tempdir isNil ifTrue:[^ false].

    cmdOut := Filename newTemporary.
    cmdOut exists ifTrue:[
        cmdOut remove.
    ].

    fullName := moduleDir , '/' , packageDir , '/' , fileName.

    "/
    "/ first, check out everything there - this creates the CVS helpfiles
    "/ required later.
    "/
    checkoutName :=  moduleDir , '/' , packageDir.

    cmd := 'cd ' , tempdir name , ';cvs checkout -l ' , checkoutName , ' > ' , cmdOut name.
    Verbose == true ifTrue:[
        ('CVSMGR: executing: ' , cmd) infoPrintNL.
    ].

    (OperatingSystem executeCommand:cmd) ifFalse:[
        'CVSMGR: failed to execute: ' errorPrint. cmd errorPrintNL.

        cmdOut remove.
        tempdir recursiveRemove.
        ('CVSMGR: cannot checkout ' , checkoutName) errorPrintNL.
        ^ false.
    ].

    "/
    "/ create the source there
    "/
    tempFile := (tempdir name , '/' , checkoutName , '/' , fileName) asFilename.
    aStream := tempFile writeStream.
    aStream isNil ifTrue:[
        ('CVSMGR: temporary fileout failed -> ', tempFile name) errorPrintNL.
        ^ false
    ].

    Class fileOutErrorSignal handle:[:ex |
        'CVSMGR: fileout failed' errorPrintNL.
        aStream close.
        ^ false
    ] do:[
        aClass fileOutOn:aStream withTimeStamp:false.
    ].
    aStream close.

    tempFile exists ifFalse:[
        'SOURCEMGR: temporary fileout failed' errorPrintNL.
        ^ false
    ].

    "/
    "/ and add it to the repository
    "/
    cmd := 'cd ' , tempdir name , '/' , checkoutName , ';cvs add ' , fileName , ' > ' , cmdOut name.
    Verbose == true ifTrue:[
        ('CVSMGR: executing: ' , cmd) infoPrintNL.
    ].

    (OperatingSystem executeCommand:cmd) ifFalse:[
        'CVSMGR: failed to execute: ' errorPrint. cmd errorPrintNL.

        cmdOut remove.
        tempdir recursiveRemove.
        ('CVSMGR: cannot checkout ' , checkoutName) errorPrintNL.
        ^ false.
    ].

    cmdOut remove.

    "/
    "/ commit
    "/
    cmd := 'cd ' , tempdir name , ';cvs commit -m "intitial checkin" 2> ' , cmdOut name.
    Verbose == true ifTrue:[
        ('CVSMGR: executing: ' , cmd) infoPrintNL.
    ].
    (OperatingSystem executeCommand:cmd) ifFalse:[
        Verbose == true ifTrue:[
            'CVSMGR: failed to execute: ' infoPrint. cmd infoPrintNL.
        ].
        whatHappened := cmdOut contentsOfEntireFile asString.
        self warn:'The following problem was reported by cvs:

' , whatHappened , '

The class has NOT been checked into the repository.'.

       'CVSMGR: cannot checkin modified class source' errorPrintNL.
        cmdOut remove.
        tempdir recursiveRemove.
        ^ false.
    ].
    whatHappened := cmdOut contentsOfEntireFile asString.

    "/
    "/ release it towards cvs
    "/
    self releaseAndRemove:tempdir outputTo:nil. 

    "/
    "/ good - its in the CVS repository; now, we need the updated RCS header
    "/
    (whatHappened isNil or:[whatHappened isEmpty]) ifTrue:[
        'CVSMGR: oops - unexpected empty commit command output' errorPrintNL.
    ] ifFalse:[
        whatHappened := whatHappened asCollectionOfLines asStringCollection.
        idx := whatHappened indexOfLineStartingWith:'initial revision:'.
        idx ~~ 0 ifTrue:[
            startIdx := 18
        ] ifFalse:[
            idx := whatHappened indexOfLineStartingWith:'new revision:'.
            idx ~~ 0 ifTrue:[
                'CVSMGR: oops - container existed' errorPrintNL.
                startIdx := 14.
            ] ifFalse:[
                'CVSMGR: oops - unexpected commit command output (no new-revision info)' errorPrintNL.
            ]
        ].
        idx ~~ 0 ifTrue:[
            l := whatHappened at:idx.
            newRevision := (l copyFrom:startIdx to:(l indexOf:$; startingAt:startIdx)-1) withoutSpaces.
        ]
    ].

    "/
    "/ patch the classes revisionInfo (but keep binaryRevision unchanged)
    "/ this makes everyone here believe, that the incore version of the class is based upon
    "/ the newly checked in version.
    "/ (however, the binaryRevision must remain as it is - we will need it to fetch the sourceCode
    "/  correctly for all unchanged methodss)
    "/

    newRevision notNil ifTrue:[
        changeLog := self 
                        revisionLogOf:aClass 
                        fromRevision:newRevision 
                        toRevision:newRevision
                        fileName:fileName
                        directory:packageDir
                        module:moduleDir.

        (changeLog isNil or:[(changeLog at:#revisions) size ~~ 1]) ifTrue:[
            'CVSMGR: oops - failed to update revisionString (no log)' errorPrintNL
        ] ifFalse:[
            entry := (changeLog at:#revisions) first.
            newString := self revisionStringFromLog:changeLog entry:entry.
            aClass updateVersionMethodFor:newString.
            ('CVSMGR: updated revisionString to:',newString) errorPrintNL
        ]
    ].

    tempdir recursiveRemove.
    ^ true

    "Created: 9.12.1995 / 19:13:37 / cg"
    "Modified: 17.12.1995 / 16:02:06 / cg"
!

createModule:moduleName
    |tempdir moduleDir cmdOut cmd|

    "/
    "/ first, create a temporary work tree
    "/
    tempdir := self createTempDirectory:nil forModule:moduleName.
    tempdir isNil ifTrue:[^ false].

    cmdOut := Filename newTemporary.
    cmdOut exists ifTrue:[
        cmdOut remove.
    ].

    "/
    "/ create the module directory 
    "/
    moduleDir := tempdir construct:moduleName.
    moduleDir exists ifFalse:[
        'CVSMGR: failed to create: ' errorPrint.  moduleDir name errorPrintNL.
        cmdOut remove.
        tempdir recursiveRemove.
        ^ false.
    ].     

    "/
    "/ then import it into CVS
    "/
    cmd := 'cd ' , tempdir name , ';cvs import -m "created from browser" ' , '.' , ' initial initial > ' , cmdOut name.
    Verbose == true ifTrue:[
        ('CVSMGR: executing: ' , cmd) infoPrintNL.
    ].

    (OperatingSystem executeCommand:cmd) ifFalse:[
        'CVSMGR: failed to execute: ' errorPrint. cmd errorPrintNL.

        cmdOut remove.
        tempdir recursiveRemove.
        ^ false.
    ].

    'CVSMGR: created new module: ' infoPrint. moduleName errorPrintNL.
    tempdir recursiveRemove.
    ^ true

    "Created: 9.12.1995 / 19:53:51 / cg"
    "Modified: 11.12.1995 / 13:40:59 / cg"
!

createModule:moduleDirName package:packageDirName
    |tempdir cmdOut cmd packageDir moduleDir|

    "/
    "/ first, create a temporary work tree
    "/
    tempdir := self createTempDirectory:nil forModule:moduleDirName.
    tempdir isNil ifTrue:[^ false].

    cmdOut := Filename newTemporary.
    cmdOut exists ifTrue:[
        cmdOut remove.
    ].

    "/
    "/ first checkout that module directory (to let CVS create its files)
    "/
    cmd := 'cd ' , tempdir name , ';cvs checkout -l ' , moduleDirName , ' > ' , cmdOut name.
    Verbose == true ifTrue:[
        ('CVSMGR: executing: ' , cmd) infoPrintNL.
    ].

    (OperatingSystem executeCommand:cmd) ifFalse:[
        'CVSMGR: failed to execute: ' errorPrint. cmd errorPrintNL.

        cmdOut remove.
        tempdir recursiveRemove.
        ^ false.
    ].

    moduleDir := tempdir construct:moduleDirName.
    moduleDir isDirectory ifFalse:[
        'CVSMGR: command failed: ' errorPrint. cmd errorPrintNL.

        cmdOut remove.
        tempdir recursiveRemove.
        ^ false.
    ].
        
    "/
    "/ create the package directory there
    "/
    packageDir := moduleDir construct:packageDirName.
    OperatingSystem errorSignal catch:[
        packageDir makeDirectory.
    ].
    packageDir isDirectory ifFalse:[
        cmdOut remove.
        tempdir recursiveRemove.
        ('CVSMGR: cannot create new directory') errorPrintNL.
        ^ false.
    ].

    "/
    "/ let cvs add this directory ...
    "/
    cmd := 'cd ' , moduleDir name , ';cvs add ' , packageDirName , ' > ' , cmdOut name.
    Verbose == true ifTrue:[
        ('CVSMGR: executing: ' , cmd) infoPrintNL.
    ].

    (OperatingSystem executeCommand:cmd) ifFalse:[
        'CVSMGR: failed to execute: ' errorPrint. cmd errorPrintNL.

        cmdOut remove.
        tempdir recursiveRemove.
        ('CVSMGR: cannot add ' , packageDirName) errorPrintNL.
        ^ false.
    ].

    cmdOut remove.

    self releaseAndRemove:tempdir outputTo:nil. 

    ^ true

    "Created: 9.12.1995 / 20:09:33 / cg"
    "Modified: 11.12.1995 / 13:42:48 / cg"
!

revisionLogOf:cls fromRevision:rev1 toRevision:rev2 fileName:classFileName directory:packageDir module:moduleDir
    "return info about the repository container and
     (part of) the revisionlog as a collection of revision entries.
     Return nil on failure.

     The returned information is a structure (IdentityDictionary)
     filled with:
            #container          -> the RCS container file name 
            #filename           -> the actual source file name
            #newestRevision     -> the revisionString of the newest revision
            #numberOfRevisions  -> the number of revisions in the container
            #revisions          -> collection of per-revision info (see below)

            rev1 / rev2 specify from which revisions a logEntry is wanted:
             -If rev1 is nil, the first revision is the initial revision
              otherwise, the log starts with that revision.
             -If rev2 is nil, the last revision is the newest revision
              otherwise, the log ends with that revision.

             -If both are nil, all logEntries are extracted.
             -If both are 0 (not nil), no logEntries are extracted (i.e. only the header).

            per revision info consists of one record per revision:

              #revision              -> the revision string
              #author                -> who checked that revision into the repository
              #date                  -> when was it checked in
              #state                 -> the RCS state
              #numberOfChangedLines  -> the number of changed line w.r.t the previous

            revisions are ordered newest first 
            (i.e. the last entry is for the initial revision; the first for the most recent one)
        "

    |tempDir cmd fullName modulePath inStream inHeaderInfo atEnd line revArg revLine1 revLine2 idx
     info record revisionRecords s headerOnly|

    CVSRoot isNil ifTrue:[
        'CVSMGR: CVSROOT not set' infoPrintNL.
        ^ nil
    ].

    modulePath :=  moduleDir , '/' , packageDir. 
    fullName :=  modulePath , '/' , classFileName.
    tempDir := self createTempDirectory:nil forModule:nil.
    [
        self createEntryFor:fullName 
             in:(tempDir construct:modulePath) 
             revision:'1.1' 
             date:'dummy' 
             special:''
             overwrite:false.

        revArg := ''.
        headerOnly := false.
        (rev1 notNil or:[rev2 notNil]) ifTrue:[
            (rev1 == 0 and:[rev2 == 0]) ifTrue:[
                revArg := '-h'.
                headerOnly := true.
            ] ifFalse:[
                revArg := '-r'.
                rev1 notNil ifTrue:[
                    revArg := revArg , rev1
                ].
                revArg := revArg , ':'.
                rev2 notNil ifTrue:[
                    revArg := revArg , rev2
                ].
            ]
        ].

        self activityNotification:'reading log'.
        cmd := 'cd ', tempDir name, '; cvs -d ', CVSRoot, ' log ' , revArg , ' ' , fullName.
    "/ cmd printNL.
        inStream := PipeStream readingFrom:cmd.
        inStream isNil ifTrue:[
            ('CVSMGR: cannot open pipe to cvs log ', fullName) infoPrintNL.
            ^ nil
        ].

        "/
        "/ read the commands pipe output and extract the container info
        "/
        info := IdentityDictionary new.
        inHeaderInfo := true.
        [inHeaderInfo and:[inStream atEnd not]] whileTrue:[
            line:= inStream nextLine.
            line notNil ifTrue:[
                |gotIt|

                gotIt := false.
                #('RCS file:' #container
                  'Working file:' #filename
                  'head:' #newestRevision
                  'total revisions:' #numberOfRevisions
                 ) pairWiseDo:[:word :key |
                    gotIt ifFalse:[
                        s := line restAfter:word withoutSeparators:true.
                        s notNil ifTrue:[info at:key put:s. gotIt := true].                        
                    ]
                ].
                gotIt ifFalse:[
                    (line startsWith:'description:') ifTrue:[inHeaderInfo := false].
                ]
            ]
        ].
        inStream nextLine. "/ skip separating line after description.

        "/ strip selected revisions from the total-revisions entry
        s := info at:#numberOfRevisions.
        (idx := s indexOf:$;) ~~ 0 ifTrue:[
            info at:#numberOfRevisions put:(Integer readFrom:(s copyTo:idx - 1))
        ].

        headerOnly ifFalse:[
            "/
            "/ continue to read the commands pipe output 
            "/ and extract revision info records
            "/
            revisionRecords := OrderedCollection new:(info at:#numberOfRevisions).
            info at:#revisions put:revisionRecords.

            atEnd := false.
            [atEnd or:[inStream atEnd]] whileFalse:[
                revLine1 := inStream nextLine.
                [revLine1 notNil and:[(revLine1 startsWith:'revision ') not]]
                    whileTrue:[revLine1 := inStream nextLine.].
                revLine2 := inStream nextLine.
                (revLine1 notNil and:[revLine2 notNil]) ifTrue:[
                    record := IdentityDictionary new.
                    record at:#revision put:(revLine1 asCollectionOfWords at:2).
                    "/ decompose date/author/state etc.
                    (revLine2 asCollectionOfSubstringsSeparatedBy:$;) do:[:info |
                        |subEntry|
                        subEntry := info withoutSeparators.
                        #('date:' #date
                          'author:' #author 
                          'state:' #state 
                          'lines:' #numberOfChangedLines
                         ) pairWiseDo:[:word :key |
                            s := subEntry restAfter:word withoutSeparators:true.
                            s notNil ifTrue:[record at:key put:s.].                        
                        ].
                    ].

                    s := nil.
                    line := inStream nextLine.
                    [atEnd or:[line isNil or:[line startsWith:'--------']]] whileFalse:[
                        (line startsWith:'==========') ifTrue:[
                            atEnd := true.
                        ] ifFalse:[
                            (line withoutSpaces = '.') ifTrue:[
                                line := '*** empty log message ***'
                            ].
                            s isNil ifTrue:[
                                s := line
                            ] ifFalse:[
                                s := s , Character cr asString , line.
                            ].
                            line := inStream nextLine.
                        ]
                    ].
                    record at:#logMessage put:s.
                    revisionRecords add:record.
                ]
            ].
        ].
        inStream close.
    ] valueNowOrOnUnwindDo:[
        tempDir recursiveRemove
    ].
    ^ info

    "
     SourceCodeManager revisionLogOf:Array 
     SourceCodeManager revisionLogOf:Array fromRevision:'1.40' toRevision:'1.43' 
     SourceCodeManager revisionLogOf:Array fromRevision:'1.40' toRevision:nil  
     SourceCodeManager revisionLogOf:Array fromRevision:nil toRevision:'1.3'  
     SourceCodeManager revisionLogOf:Array fromRevision:nil toRevision:nil  
     SourceCodeManager revisionLogOf:Array fromRevision:0 toRevision:0  
    "

    "Created: 16.11.1995 / 13:25:30 / cg"
    "Modified: 16.12.1995 / 18:53:02 / cg"
    "Modified: 20.12.1995 / 11:06:11 / stefan"
!

writeHistoryLogSince:timeGoal filterSTSources:filter filterUser:userFilter to:aStream
    |cmd inStream line words fileName user date time rev pkgDir clsName cls clsRev|

    CVSRoot isNil ifTrue:[
        'CVSMGR: CVSROOT not set' infoPrintNL.
        ^ nil
    ].
    cmd := 'cvs history -a -report -c -D ' , timeGoal.
"/ cmd printNL.
    inStream := PipeStream readingFrom:cmd.
    inStream isNil ifTrue:[
        ('CVSMGR: cannot open pipe to cvs history') infoPrintNL.
        ^ nil
    ].

    aStream nextPutLine:'**** source repository history since ' , timeGoal , ' ****'.
    aStream cr.
    aStream cr.
    aStream nextPutAll:' date  time'; tab; nextPutAll:'      user'; tab; 
            nextPutAll:'          file'; tab ; nextPutAll:'     rev'; tab ; nextPutAll:'module/package'; cr.
    aStream cr.

    "/
    "/ read the commands pipe output and reformat the lines
    "/
    [inStream atEnd not] whileTrue:[
        line := inStream nextLine.
        line notNil ifTrue:[
            "/ cvs history line is of the following format:
            "/ [Ma] date time +xxx user rev name module/package ....
            words := line asCollectionOfWords.
            words size >= 8 ifFalse:[
                "/ something like 'No records selected' ...
                aStream nextPutLine:line
            ] ifTrue:[
                fileName := words at:7.
                user := words at:5.
                (filter not 
                or:[fileName endsWith:'.st']) ifTrue:[
                    (userFilter isNil
                    or:[userFilter = user]) ifTrue:[
                        date := words at:2.
                        time := words at:3.
                        rev := words at:6.
                        pkgDir := words at:8.

                        aStream nextPutAll:date; space; nextPutAll:time; tab;
                                nextPutAll:(user leftPaddedTo:10); tab;             
                                nextPutAll:(fileName leftPaddedTo:14); tab;                 
                                nextPutAll:(rev decimalPaddedTo:8 and:3 at:$. withLeft:(Character space) right:nil); tab;              
                                nextPutAll:(pkgDir paddedTo:20).

                        "/
                        "/ for your convenience:
                        "/  check what the actual version is in the image
                        "/
                        clsName := Smalltalk classNameForFile:fileName.
                        clsName notNil ifTrue:[
                            cls := Smalltalk classNamed:clsName.
                            (cls notNil and:[(clsRev := cls revision) notNil]) ifTrue:[
                                rev ~= clsRev ifTrue:[
                                    aStream nextPutAll:('current: ' , (clsRev decimalPaddedTo:8 and:3 at:$. withLeft:(Character space) right:nil))
                                ]
                            ]
                        ].
                        aStream cr                              
                    ]
                ]
            ]
        ]
    ].
    inStream close.

    "Created: 13.12.1995 / 10:28:27 / cg"
    "Modified: 23.12.1995 / 16:36:43 / cg"
! !

!CVSSourceCodeManager class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/libbasic3/Attic/CVSSCMgr.st,v 1.75 1996-01-06 14:38:07 cg Exp $'
! !
CVSSourceCodeManager initialize!
