"
 COPYRIGHT (c) 1995 by AEG Industry Automation
 COPYRIGHT (c) 1995 by Claus Gittinger
	      All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"

Object subclass:#HistoryLine
	instanceVariableNames:'date time user firstPositionInSourceCode type'
	classVariableNames:'Quote Seperator ModifiedString DeletedString IndentString
		UseGECOS CreatedString'
	poolDictionaries:''
	category:'System-Changes-History'
!

!HistoryLine class methodsFor:'documentation'!

copyright 
"
 COPYRIGHT (c) 1995 by AEG Industry Automation
 COPYRIGHT (c) 1995 by Claus Gittinger
	      All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"
!

documentation
"
    The class HistoryLine is part of the
	HistoryManagerProjcet.
    It creates instaces for handling multi user development in the Smalltalk environment.

    HistoryLine knows how to compose and parse comment lines which are
    used to keep track of changes. These historyLines are added at the end of
    a methods source code and to the special history classmethod.
    HistoryLine and HistoryManager where generously provided by AEG for inclusion into the
    ST/X class delivery.

    Author: Robert Sailer - AEG
"
!

examples
"
    HistoryLine initialize.
    HistoryLine new.
    HistoryLine for: 'R.Sailer'.    for integration purposes ONLY
    HistoryLine deleted.
"
!

history
    "Created: 23.08.1995 / 21:48:18 / robert"

    "Modified: 23.08.1995 / 21:48:19 / robert"
! !

!HistoryLine class methodsFor:'initialization'!

initialize

    Quote := '"'.
    Seperator := ' / '.
    ModifiedString := 'Modified: '.
    DeletedString := 'Deleted: '.
    CreatedString := 'Created: '.
    IndentString := '    '.
    UseGECOS := false.

    "
     HistoryLine initialize
    "

    "Modified: 23.08.1995 / 22:14:03 / robert"
! !

!HistoryLine class methodsFor:'instance creation'!

fromString: aString
    ^self fromString: aString at: 0.

    "
     HistoryLine fromString: 'Modified: 01.12.93 / 18:32:30 / M.Noell'
     HistoryLine fromString: 'Created: 01.12.93 / 18:32:30 / M.Noell'
    "

    "Modified: 23.08.1995 / 22:14:13 / robert"
!

fromString: aString at: position
    "parses the String and creates a new Instance with the values. 
     The positionvalue is normally used to remove the HistoryLines from the sourceCode.
     Claus: return nil, if the string is not a valid historyString."

    | inst anArray  aTime aDate userName initial |

    inst := super new.  "because the local new initializes the new object - not wanted here"

    anArray := aString asArrayOfSubstrings.
    anArray size < 6 ifTrue:[^ nil].

    "
	1 = Modified / Deleted / Created
	2 = date asString
	3 = Separator
	4 = time asString
	5 = Separator
	6 = UserName...
    "

    initial := anArray first withoutSpaces.
    initial = 'Modified:' ifTrue:[
	"I think it is the text Modified"
	inst type: ModifiedString.
    ] ifFalse:[
	initial = 'Deleted:'  ifTrue: [
	    "now it should be deleted"
	    inst type: DeletedString.
	] ifFalse:[
	    initial = 'Created:'  ifTrue: [
		"now it should be created"
		inst type: CreatedString.
	    ] ifFalse:[
		^ nil
	    ]
	]
    ].

"    aDate := Date readFromString: (anArray at: 2) onError:[ Date today ]."
"    aTime := Time readFromString: (anArray at: 4) onError: [ Time now  ]."
    aDate := self convertAStringToADate: (anArray at: 2).
    aTime := self convertAStringToATime: (anArray at: 4).


    inst date: aDate.
    inst time: aTime.
    anArray size >= 6 ifTrue:[
	"the user's name must be more that one word"
	userName := ''.
	anArray from: 6 to: anArray size  do:[:anElem|
	    userName := userName,' ', anElem.
	].
	userName := userName withoutSpaces
    ].
    inst user: userName.
    inst firstPositionInSourceCode: position.

    ^inst

    "
     HistoryLine fromString: 'Modified: 21.12.93 / 18:32:30 / R.Sailer' at: 199
     HistoryLine fromString: 'Created: 21.12.1993 / 18:32:30 / Felicitas Gabriele Felger' at: 199
     HistoryLine fromString: 'Deleted: 21.12.93 / 18:32:30 / Astrid Weieise' at: 199
     HistoryLine fromString: 'foo bar Deleted: 21.12.93 / 18:32:30 / Astrid Weieise' at: 199
    "

    "Modified: 23.8.1995 / 22:24:47 / robert"
    "Modified: 19.9.1995 / 14:14:48 / claus"
!

new
    ^ self newModified
!

newCreated
    "public - get a created-Line for the actual user"    

    ^ self createdBy:(self currentUserName).

    "
	HistoryLine created
    "

    "Modified: 23.08.1995 / 22:14:24 / robert"
!

newDeleted
    "public - get a deleted-Line for the actual user"    

    ^ self deletedBy:(self currentUserName).

    "Modified: 23.08.1995 / 22:14:22 / robert"
!

newModified
    "create a preinitialized HistoryLine filled with defaults"
    | inst |
    inst := super new.

    inst date: Date today.
    inst time: Time now.
    inst user:(self currentUserName).
    inst firstPositionInSourceCode: 0.
    inst type: ModifiedString.
    ^inst

    "Modified: 23.08.1995 / 21:49:51 / robert"
! !

!HistoryLine class methodsFor:'converting'!

convertAStringToADate: aString
   "kludge while not having the Time and Date format spec Class"

    | day  month year coll |

    "delete delimiter from the date string"
    aString replaceAll: $. by: $ .
    coll := aString asArrayOfSubstrings.
    day := (coll at: 1) asNumber.
    month := (coll at: 2 ) asNumber.
    year := (coll at: 3 ) asNumber.

    ^Date day: day month: month year: year.

    "
     HistoryLine convertAStringToADate:'18.10.1995'
     HistoryLine convertAStringToADate:'18.10.95'
    "

    "Modified: 23.08.1995 / 21:28:58 / robert"
!

convertAStringToATime: aString
   "kludge while not having the Time and Date format spec Class"

    | h m s|

    h := (aString copyFrom: 1 to: 2) asNumber.
    m := (aString copyFrom: 4 to: 5) asNumber.
    s := (aString copyFrom: 7 to: 8) asNumber.


    ^Time hour:h  minutes: m seconds: s.        

    "
     HistoryLine convertAStringToATime:'18:23:15' 
    "

    "Modified: 15.08.1995 / 18:56:18 / robert"
! !

!HistoryLine class methodsFor:'filtering'!

filterHistoryLines:  aCollectionOfHistoryLines
    "check the oc against multiple occurrence of the same user and take only the youngest."

    |newCollection|

    newCollection := OrderedCollection new.
    aCollectionOfHistoryLines keysAndValuesDo:[:index :histLine |
        |skip|

        skip := false.
        histLine isModified ifTrue:[
            "/ if there is another one, skip this
            aCollectionOfHistoryLines from:index+1 do:[:anotherHistLine |
                anotherHistLine isModified ifTrue:[
                    anotherHistLine user = histLine user ifTrue:[
                        skip := true
                    ]
                ]
            ].
        ] ifFalse:[
            "/
            "/ filter out multiple created messages
            "/ (this only occurs if a method was copied from ClassA to ClassB)
            "/
            histLine isCreated ifTrue:[
                aCollectionOfHistoryLines from:index+1 do:[:anotherHistLine |
                    anotherHistLine isCreated ifTrue:[
                        skip := true
                    ]
                ].
            ].
        ].
        skip ifFalse:[
            newCollection add:histLine.
        ]
    ].
    ^ newCollection.

"/    | allUsers newCollection aHistLine |
"/
"/    allUsers := Set new.
"/
"/    anOrderdCollectionOfHistoryLines do:[ :hl| allUsers add: hl user ].
"/
"/    allUsers size ~~ anOrderdCollectionOfHistoryLines ifTrue:[
"/        "there is at least one user twice"
"/        newCollection := OrderedCollection new.
"/        1 to: (anOrderdCollectionOfHistoryLines size - 1) do:[ :index|
"/            aHistLine :=(anOrderdCollectionOfHistoryLines at: index).
"/            allUsers last = aHistLine user ifFalse:[
"/                newCollection add: aHistLine.
"/            ].
"/        ].
"/        "add the last element with the actucal HistoryLine"
"/        newCollection add: anOrderdCollectionOfHistoryLines last.
"/
"/
"/        "the new collection shold now be sorted at timeStamp. --- for further study"
"/
"/        ^newCollection.
"/    ].
"/
"/    ^ anOrderdCollectionOfHistoryLines


    "
     |u1 u2 u3 u4 u5 oc |
     u1 := HistoryLine new.
     u2 := HistoryLine for: 'claus'.
     u3 := HistoryLine for: 'chris'.
     u4 := HistoryLine new.
     u5 := HistoryLine new.

     oc := OrderedCollection new.
     oc add: u1.
     oc add: u2.
     oc add: u3.
     oc add: u4.
     oc add: u5.

     oc inspect.
     (HistoryLine filterHistoryLines: oc) inspect 
    "

    "Modified: 8.9.1995 / 17:20:40 / claus"
    "Modified: 13.12.1995 / 14:10:03 / cg"
! !

!HistoryLine class methodsFor:'private'!

createdBy: aUserName
    "private - for integration purposes only"

    | inst |
    inst := self new.

    inst user: aUserName.
    inst type: CreatedString.    

    ^inst

    "Modified: 23.08.1995 / 21:35:44 / robert"
!

currentUserName
    "return the current users name - 
     thats either the userInfos-gecos field, or the users login name."

    | info gecos|

    UseGECOS = true ifTrue:[
	info := OperatingSystem userInfoOf:(OperatingSystem getUserID).
	(info notNil
	and:[info includesKey:#gecos]) ifTrue:[
	    gecos := info at:#gecos.
	    (gecos includes:$,) ifTrue:[
		^ gecos copyTo:(gecos indexOf:$,) - 1
	    ].
	    ^ gecos
	].
    ].
    ^ (OperatingSystem getLoginName).

    "Modified: 8.9.1995 / 16:59:47 / claus"
!

deletedBy: aUserName
    "private - for integration purposes only"

    | inst |
    inst := self new.

    inst user: aUserName.
    inst type: DeletedString.    

    ^inst

    "Modified: 23.08.1995 / 21:36:00 / robert"
!

for: aUserName
    "private - to create a HistoryLine for a not actually logged In user (for integration purposes only)"
    ^self modifiedBy: aUserName

    "Modified: 23.08.1995 / 21:33:01 / robert"
!

modifiedBy: aUserName
    "private - to create a HistoryLine for a not actually logged In user (for integration purposes only)"
    | inst |
    inst := self new.

    inst user: aUserName.
    inst type: ModifiedString.

    ^inst

    "Modified: 23.08.1995 / 21:35:07 / robert"
! !

!HistoryLine methodsFor:'accessing'!

date
    "return date"

    ^ date!

date:something
    "set date"

    date := something.!

firstPositionInSourceCode
    "return firstPositionInSourceCode"

    ^ firstPositionInSourceCode!

firstPositionInSourceCode:something
    "set firstPositionInSourceCode"

    firstPositionInSourceCode := something.!

time
    "return time"

    ^ time!

time:something
    "set time"

    time := something.!

type
    "return type"

    ^ type!

type:something
    "set type"

    type := something.!

user
    "return user"

    ^ user!

user:something
    "set user"

    user := something.! !

!HistoryLine methodsFor:'comparing'!

= aHistoryLine
    "compares two instances of HistoryLine"

    (aHistoryLine user = self user) ifTrue:[
	(aHistoryLine date = self date) ifTrue:[
	    (aHistoryLine time = self time) ifTrue:[
		(aHistoryLine type = self type) ifTrue:[
		    ^ true
		]
	    ]
	]
    ].

    ^ false

"
    | h1 h2 |

    h1 := HistoryLine for: OperatingSystem getLoginName.
    h2 := h1 copy.

    h1 = h2 ifTrue:[
	InfoBox new title: 'users are equal'; show.
    ].

    h2 := HistoryLine for: OperatingSystem getLoginName.
    h1 = h2 ifFalse:[
	InfoBox new title: 'users are not equal'; show.
    ].
"

    "Modified: 23.08.1995 / 22:26:40 / robert"
!

hash
    "return a hash key for the receiver"

    ^user hash        

"
    Check hashCode

    |h1 h2 oc |

    h1 := HistoryLine new hash.
    h2 := HistoryLine new hash.
    oc := OrderedCollection new.
    oc add: h1.
    oc add: h2.
    ^oc

    using hash in a set:

    | h1 h2 aSet oc |
    h1 := HistoryLine new hash.
    h2 := HistoryLine new hash.
    oc := OrderedCollection new.
    oc add: h1.
    oc add: h2.
    aSet := oc asSet.
    ^aSet
"

    "Modified: 23.08.1995 / 22:26:44 / robert"
!

sameDate: aHistoryLine
    "returns true if aUserName = user in preperation for a SortedCollection of HistoryLines"

    ^aHistoryLine date = date

"
    | h1 h2 |

    h1 := HistoryLine new.
    h2 := h1 copy.

    (h1 sameDate: h2)  ifTrue:[
	InfoBox new title: 'Dates are equal'; show.
    ].

    h2 := HistoryLine for: 'R.Sailer'.
    h2 date: (Date day: 12 month: 6 year:1981).
    (h1 sameDate: h2) ifFalse:[
	InfoBox new title: 'users are not equal'; show.
    ].

"

    "Modified: 23.08.1995 / 22:26:47 / robert"
!

sameType: aHistoryLine
    "returns true if the Type = type in preperation for a SortedCollection of HistoryLines"

    ^aHistoryLine type = type

"
    | h1 h2 |

    h1 := HistoryLine new.
    h2 := h1 copy.

    (h1 sameType: h2)  ifTrue:[
	InfoBox new title: 'Types are equal'; show.
    ].

    h2 := HistoryLine createdBy: 'R.Sailer'.
    (h1 sameType: h2) ifFalse:[
	InfoBox new title: 'Types are not equal'; show.
    ].

"

    "Modified: 23.08.1995 / 22:26:49 / robert"
!

sameUser: aHistoryLine
    "returns true if aUserName = user in preperation for a SortedCollection of HistoryLines"

    ^aHistoryLine user = user


"
    | h1 h2 |

    h1 := HistoryLine new.
    h2 := h1 copy.

    (h1 sameUser: h2)  ifTrue:[
	InfoBox new title: 'users are equal'; show.
    ].

    h2 := HistoryLine for: 'R.Sailer'.
    (h1 sameUser: h2) ifFalse:[
	InfoBox new title: 'users are not equal'; show.
    ].

"

    "Modified: 23.08.1995 / 22:26:51 / robert"
! !

!HistoryLine methodsFor:'printing & storing'!

printString
    "return representtaion of a HistoryLine"

    | aStream timeString dateString |

    aStream := ReadWriteStream on: String new.
    time print24HourFormatOn:aStream. 
    timeString := aStream contents.    

    dateString := date printFormat:#(1 2 3 $. 1 1).

    ^ IndentString, Quote, type, dateString, Seperator, timeString, Seperator, user, Quote.

    "
     HistoryLine initialize.
     | hl |

     hl := HistoryLine new .
     ^hl.
    "
! !

!HistoryLine methodsFor:'queries'!

isCreated
    "returns true if Body Text is CreatedString"

    ^type = CreatedString

"

	HistoryLine new isModified
	(HistoryLine for: 'R.Sailer') isCreated 
	(HistoryLine createdBy: 'R.Sailer') isCreated 
	HistoryLine deleted isModified 
	(HistoryLine deletedBy: 'M.Noell') isModified 
        
"

    "Modified: 23.08.1995 / 22:30:23 / robert"
!

isDeleted
    "returns true if Body Text is DeletedString"

    ^type = DeletedString

"

	HistoryLine deleted isDeleted
	HistoryLine new isDeleted



"
!

isModified
    "returns true if Body Text is ModifiedString"

    ^type = ModifiedString

"

	HistoryLine new isModified
	(HistoryLine for: 'R.Sailer') isModified 
	HistoryLine deleted isModified 
	(HistoryLine deletedBy: 'M.Noell') isModified 
        
"
! !

!HistoryLine class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/libbasic3/Attic/HistLine.st,v 1.12 1995-12-13 13:32:46 cg Exp $'
! !
HistoryLine initialize!
